/******************************************************************************
 *
 * Copyright (c) 1999 Palm Computing, Inc. or its subsidiaries.
 * All rights reserved.
 *
 * File: Starter.cpp
 *
 *****************************************************************************/

#include <PalmOS.h>
#include "SearchlightRsc.h"
#include <PalmTypes.h>
#include <Window.h>
#include <Bitmap.h>

#define SEARCHLIGHT 0
#define RHGUARD     1
#define LHGUARD     2
#define CONVICT     3
#define MISC        4

#define SOUND       3

// List of key bindings
#define moveLeftKey1		keyBitHard1		// polled every game period
#define moveLeftKey2		keyBitHard2		// polled every game period
#define moveRightKey1		keyBitHard4		// polled every game period
#define moveRightKey2		keyBitHard3		// polled every game period
#define moveLeftKeys		(moveLeftKey1 | moveLeftKey2)
#define moveRightKeys		(moveRightKey1 | moveRightKey2)
#define keysAllowedMask		(moveLeftKey1 | moveLeftKey2 | moveRightKey1 | moveRightKey2)

int game_paused;
int user_paused;
long time_of_pause;

int game_speed_factor;
int min_game_speed_factor = 20;
long last_level_change;
long score;
int farthest_position;



long time_of_soonest_event;
long time_of_next_event[4];

int direction[4];		// direction of motion for automated sprites
int position[4];		// position of sprites
int old_position[4];	// last position of sprites
int num_positions[4];	// number of possible positions (including offscreen
						// positions) for sprites
long base_interval[4];	// base # of clicks between movements for automated sprites
int resource_base[5];	// resource # of first bitmap family for sprites
int num_bitmaps[5];		// number of bitmaps to load for each sprite

int num_captured_prisoners;
int holes_dug;

unsigned int sound_volume;
int current_sound;

int in_act_of_digging = 0;

typedef struct
{
	int priority;
	int periods;
	long frequency;
	unsigned int duration;
} sound_info;

static sound_info game_sounds[6] = 
{
	{0,	0,	0, 	  0},			// no sound
	{0,	1,	700,  25},
	{0,	1,	250,  25},			
	{0, 1,  1500, 20},
	{0, 1,  100,  100},
	{0, 1,  2000, 20},
};


MemHandle				ObjectBitmapHandles[5][29];
BitmapPtr				ObjectBitmapPtr[5][29];
WinHandle				ObjectWindowHandles[5][29];

MemHandle				bgBitmapHandle;
BitmapPtr				bgBitmapPtr;
WinHandle				bgWindowHandle;

/***********************************************************************
 *
 *   Entry Points
 *
 ***********************************************************************/


/***********************************************************************
 *
 *   Internal Structures
 *
 ***********************************************************************/
typedef struct 
{
  int game_type;
  int game_difficulty;
  int sound_option;
  long high_score_a;
  long high_score_b;
  
  int demo_mode;
  int lhguard_position;
  int lhguard_direction;
  int searchlight_position;
  int searchlight_direction;
  int rhguard_position;
  int rhguard_direction;
  int convict_position;
  int farthest_position;
  int num_captured_prisoners;
  int holes_dug;
  long score;
  int game_speed_factor;
  long last_level_change;
} SearchlightPreferenceType;

SearchlightPreferenceType prefs;

typedef struct 
	{
	UInt8 replaceme;
	} StarterAppInfoType;

typedef StarterAppInfoType* StarterAppInfoPtr;


/***********************************************************************
 *
 *   Global variables
 *
 ***********************************************************************/
//static Boolean HideSecretRecords;


/***********************************************************************
 *
 *   Internal Constants
 *
 ***********************************************************************/
#define appFileCreator			'SRLT'
#define appVersionNum            0x01
#define appPrefID                0x00
#define appPrefVersionNum        0x01

// Define the minimum OS version we support (2.0 for now).
#define ourMinVersion	sysMakeROMVersion(2,0,0,sysROMStageRelease,0)


/***********************************************************************
 *
 *   Internal Functions
 *
 ***********************************************************************/


/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: This routine checks that a ROM version is meet your
 *              minimum requirement.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags that indicate if the application 
 *                                UI is initialized.
 *
 * RETURNED:    error code or zero if rom is compatible
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
	UInt32 romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
			{
			FrmAlert (RomIncompatibleAlert);
		
			// Palm OS 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion < ourMinVersion)
				{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
				}
			}
		
		return sysErrRomIncompatible;
		}

	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the current
 *              form.
 *
 * PARAMETERS:  formId - id of the form to display
 *
 * RETURNED:    void *
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void * GetObjectPtr(UInt16 objectID)
{
	FormPtr frmP;

	frmP = FrmGetActiveForm();
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}


/***********************************************************************
static void request_sound(int requested_sound)
{
	if (game_sounds[requested_sound].priority >= game_sounds[current_sound].priority)
	{
      current_sound = requested_sound;
	  time_of_next_event[SOUND] = TimGetTicks() + game_sounds[requested_sound].periods;
	}
}
 ***********************************************************************/

/***********************************************************************
static void play_sounds()
{
  SndCommandType snd_cmd;

  if (current_sound != 0)
  {
	snd_cmd.cmd = sndCmdFrqOn;
	snd_cmd.param1 = game_sounds[current_sound].frequency;
	snd_cmd.param2 = game_sounds[current_sound].duration;
	snd_cmd.param3 = sound_volume;

	SndDoCmd( 0, &snd_cmd, true);

	if (--sound_periods_remaining <= 0) current_sound = 0;
  }
}
 ***********************************************************************/


/***********************************************************************
 ***********************************************************************/
static void play_sound(int sound_number)
{
  SndCommandType snd_cmd;
  
  if(prefs.sound_option == 0 || prefs.demo_mode == 1) return;

  snd_cmd.cmd = sndCmdFrqOn;
  snd_cmd.param1 = game_sounds[sound_number].frequency;
  snd_cmd.param2 = game_sounds[sound_number].duration;
  snd_cmd.param3 = sound_volume;

  SndDoCmd( 0, &snd_cmd, true);
}



/***********************************************************************
 ***********************************************************************/
static void GameMaskKeys ()
{
	// Set the keys we poll to not generate events.  This saves cpu cycles.
	KeySetMask(~keysAllowedMask);
	
	// Avoid the code below because KeyRates is broken.
	return;
	
	// Also set the key repeat rate low to avoid constantly checking them.
	UInt16 initDelay = slowestKeyDelayRate;
	UInt16 period = slowestKeyPeriodRate;
	Boolean queueAhead = false;
	KeyRates(true, &initDelay, &period, &period, &queueAhead);
}


/***********************************************************************
 ***********************************************************************/
static void GameUnmaskKeys ()
{
	// Set the keys we poll to not generate events.  This saves cpu cycles.
	KeySetMask(keyBitsAll);
	
	// Avoid the code below because KeyRates is broken.
	return;
	
	// Also set the key repeat rate low to avoid constantly checking them.
	//KeyRates(true, &KeyRate.initDelay, &KeyRate.period, &KeyRate.doubleTapDelay, 
	//	&KeyRate.queueAhead);
}


/***********************************************************************
 *
 * FUNCTION:    MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:  frm - pointer to the MainForm form.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void MainFormInit(FormPtr /*frmP*/)
{
}


/***********************************************************************
 ***********************************************************************/
static void find_soonest_event(void)
{
  // find the next event scheduled to occur
  time_of_soonest_event = time_of_next_event[0];
  for(int i = 1; i < 4; i++)
  {
    if(time_of_next_event[i] > 0
       && time_of_next_event[i] < time_of_soonest_event)
      time_of_soonest_event = time_of_next_event[i];
  }  
  
  /*
  static char msg_string[255];
  StrPrintF(msg_string, "Next event in %d ticks", time_of_soonest_event - TimGetTicks());
  FrmCustomAlert(GenericErrorAlert, msg_string, NULL, NULL);
  */
}


/***********************************************************************
 ***********************************************************************/
static void back_off_events(int num_ticks)
{
  for(int i = 0; i < 4; i++)
  {
    if(time_of_next_event[i] > 0) time_of_next_event[i] += num_ticks;
  }
  find_soonest_event();
}


/***********************************************************************
 ***********************************************************************/
static void DrawBackground()
{
  RectangleType srcR;

  ErrFatalDisplayIf (bgWindowHandle == 0, "Unhandled object image");

  // Copy the entire source window.
  MemMove (&srcR, &(bgWindowHandle->windowBounds), sizeof(RectangleType));

  // Copy the source window (contains the image to draw) to the draw window.
  WinCopyRectangle(bgWindowHandle, 0, &srcR, 1, 16, winPaint);
}


/***********************************************************************
 ***********************************************************************/
static void DrawObject(int spriteNumber, int bitmapNumber, short x, short y, WinDrawOperation mode)
{
  RectangleType srcR;

  ErrFatalDisplayIf (ObjectWindowHandles[spriteNumber][bitmapNumber] == 0, "Unhandled object image");

  // Copy the entire source window.
  MemMove (&srcR, &(ObjectWindowHandles[spriteNumber][bitmapNumber]->windowBounds), sizeof(RectangleType));

  // Copy the source window (contains the image to draw) to the draw window.
  WinCopyRectangle(ObjectWindowHandles[spriteNumber][bitmapNumber], 0, &srcR, x, y, mode);
}


/***********************************************************************
 ***********************************************************************/
static void reset_convict(void)
{
  old_position[CONVICT] = position[CONVICT];
  position[CONVICT] = 0;
  holes_dug = 0;
  farthest_position = 0;
  
  // wipe out the holes in the wall
  DrawObject(MISC, 0, 25, 30, winPaint);
}


/***********************************************************************
 ***********************************************************************/
static long get_next_interval(int sprite_num, long last_time, int current_position)
{
  if(sprite_num == SEARCHLIGHT)
  {
    return last_time + base_interval[SEARCHLIGHT];
  }
  else if(sprite_num == LHGUARD)
  {
    if(current_position == 4) return last_time + base_interval[LHGUARD] * 2;
    else return last_time + base_interval[LHGUARD];
  }
  else if(sprite_num == RHGUARD)
  {
    // if guard is in position 0 (offscreen), then the time to the next
    // position is a random number between 0 ticks and 2 times the base_interval
    if(current_position == 0) 
    {
      long rand_number = (long)SysRandom(TimGetTicks());
      long rand_interval = (base_interval[RHGUARD] * (long)450 * rand_number) / ((long)sysRandomMax * (long)100);
      
      return last_time + rand_interval + base_interval[RHGUARD];
    } 
    else 
      return last_time + base_interval[RHGUARD];
  }
  // if we get called with sprite_num = CONVICT, we return 0, since there's
  // really nothing automated happening with the convict (we just artificially
  // set time_of_next_event[CONVICT] when we want to animate the digging.
  else if(sprite_num == CONVICT)
  {
    return 0;
  }
  
  return 0;
}


/***********************************************************************
 ***********************************************************************/
static void reset_positions(void)
{
  reset_convict();
      
  long current_time = TimGetTicks();
  for(int i = 0; i < 3; i++)
  {
    direction[i] = 1;
    old_position[i] = position[i];
    position[i] = 0;
    time_of_next_event[i] = get_next_interval(i, current_time, position[i]);
  }
  time_of_next_event[CONVICT] = 0;
  find_soonest_event();
}



/***********************************************************************
 ***********************************************************************/
static void recalculate_intervals(void)
{
  base_interval[SEARCHLIGHT] = ((long)sysTicksPerSecond * (long)58 * (long)game_speed_factor) / (long)10000;
  base_interval[RHGUARD] = ((long)sysTicksPerSecond * (long)game_speed_factor) / 100;
  base_interval[LHGUARD] = ((long)sysTicksPerSecond * (long)78 * (long)game_speed_factor) / 10000;
}


/***********************************************************************
 ***********************************************************************/
static void toggle_pause(void)
{
  if(game_paused == 1)
  {
    // set label of menu item to "Pause"
    //FrmSetMenu(FrmGetFormPtr(MainForm), MainFormMenuBar);
    
    game_paused = 0;
    back_off_events((int)(TimGetTicks() - time_of_pause));
  }
  else
  {
    // set label of menu item to "Resume"
    //FrmSetMenu(FrmGetFormPtr(MainForm), MainForm2MenuBar);

    game_paused = 1;
    time_of_pause = TimGetTicks();
  }
}



/***********************************************************************
 ***********************************************************************/
static long TimeUntilNextPeriod (void)
{
  long timeRemaining = time_of_soonest_event - TimGetTicks();
  if (timeRemaining < 0) timeRemaining = 0;
		
  return timeRemaining;
}





/***********************************************************************
 ***********************************************************************/
static void DrawBoard(void)
{
  /*
  long begin_time = TimGetTicks();
  static char msg_string[255];
  StrPrintF(msg_string, "positions = (%d,%d,%d,%d)", position[LHGUARD], position[SEARCHLIGHT], position[RHGUARD], position[CONVICT]);
  FrmCustomAlert(GenericErrorAlert, msg_string, NULL, NULL);
  long push_back = TimGetTicks() - begin_time;
  back_off_events((int)push_back);
*/

  static char score_string[64];
  if(prefs.demo_mode == 1 && prefs.game_type == 0)
  {
    StrPrintF(score_string, "%6d", (int)prefs.high_score_a);
  }
  else if(prefs.demo_mode == 1 && prefs.game_type == 1)
  {
    StrPrintF(score_string, "%6d", (int)prefs.high_score_b);
  }
  else
  {
//    StrPrintF(score_string, "%6d (%d)", (int)score, (int)game_speed_factor);
    StrPrintF(score_string, "%6d", (int)score);
  }  

  if(position[SEARCHLIGHT] != old_position[SEARCHLIGHT])
  {
    if(old_position[SEARCHLIGHT] == 0)      DrawObject(SEARCHLIGHT, 6, 87, 42, winPaint);
    else if(old_position[SEARCHLIGHT] == 1) DrawObject(SEARCHLIGHT, 7, 86, 42, winPaint);
    else if(old_position[SEARCHLIGHT] == 2)
    {
      DrawObject(SEARCHLIGHT, 8, 86, 42, winPaint);
      if(position[CONVICT] == 4) DrawObject(CONVICT, 4, 83, 81, winPaint);  // unstomp
    }
    else if(old_position[SEARCHLIGHT] == 3) DrawObject(SEARCHLIGHT, 9, 80, 42, winPaint);
    else if(old_position[SEARCHLIGHT] == 4)
    {
      DrawObject(SEARCHLIGHT, 10, 62, 42, winPaint);
      if(position[CONVICT] == 4) DrawObject(CONVICT, 4, 83, 81, winPaint);  // unstomp
    }  
    else if(old_position[SEARCHLIGHT] == 5)
    {
      DrawObject(SEARCHLIGHT, 11, 46, 42, winPaint);
      if(position[CONVICT] == 7) DrawObject(CONVICT,  7, 35, 40, winPaint);  // unstomp
      if(position[CONVICT] == 8) DrawObject(CONVICT, 27, 35, 40, winPaint);  // unstomp
      if(position[CONVICT] == 9) DrawObject(CONVICT, 28, 35, 40, winPaint);  // unstomp
    }
    
    
    if(position[SEARCHLIGHT] == 0)
    {
      DrawObject(SEARCHLIGHT, 0, 87, 42, winPaint);
      if(position[CONVICT] == 1) DrawObject(CONVICT, 20, 125, 51, winPaint);  // light convict
    }
    else if(position[SEARCHLIGHT] == 1) DrawObject(SEARCHLIGHT, 1, 86, 42, winPaint);
    else if(position[SEARCHLIGHT] == 2)
    {
      DrawObject(SEARCHLIGHT, 2, 86, 42, winPaint);
      if(position[CONVICT] == 3) DrawObject(CONVICT, 21, 99, 76, winPaint);     // light convict
      else if(position[CONVICT] == 4) DrawObject(CONVICT, 4, 83, 81, winPaint); // unstomp
    }  
    else if(position[SEARCHLIGHT] == 3) 
    {
      DrawObject(SEARCHLIGHT, 3, 80, 42, winPaint);
      if(position[CONVICT] == 4) DrawObject(CONVICT, 22, 83, 81, winPaint); // light convict
    }  
    else if(position[SEARCHLIGHT] == 4)
    {
      DrawObject(SEARCHLIGHT, 4, 62, 42, winPaint);
      if(position[CONVICT] == 5) DrawObject(CONVICT, 23, 64, 77, winPaint);  // light convict
      else if(position[CONVICT] == 4) DrawObject(CONVICT, 4, 83, 81, winPaint);  // unstomp
    } 
    else if(position[SEARCHLIGHT] == 5)
    {
      DrawObject(SEARCHLIGHT, 5, 46, 42, winPaint);
      if(position[CONVICT] == 7) DrawObject(CONVICT, 26, 48, 53, winPaint);  // light convict
      if(position[CONVICT] == 8) DrawObject(CONVICT, 27, 35, 40, winPaint);  // unstomp
      if(position[CONVICT] == 9) DrawObject(CONVICT, 28, 35, 40, winPaint);  // unstomp
    }
    old_position[SEARCHLIGHT] = position[SEARCHLIGHT];
  }
  
  if(position[RHGUARD] != old_position[RHGUARD])
  {
    if(old_position[RHGUARD] == 1)      DrawObject(RHGUARD, 5, 104, 103, winPaint);
    else if(old_position[RHGUARD] == 2) DrawObject(RHGUARD, 6, 125, 97, winPaint);
    else if(old_position[RHGUARD] == 3) DrawObject(RHGUARD, 7, 135, 74, winPaint);
    else if(old_position[RHGUARD] == 4) DrawObject(RHGUARD, 8, 141, 46, winPaint);
    else if(old_position[RHGUARD] == 5) DrawObject(RHGUARD, 9, 147, 26, winPaint);

    if(position[RHGUARD] == 1)      DrawObject(RHGUARD, 0, 104, 103, winPaint);
    else if(position[RHGUARD] == 2) DrawObject(RHGUARD, 1, 125, 97, winPaint);
    else if(position[RHGUARD] == 3) DrawObject(RHGUARD, 2, 135, 74, winPaint);
    else if(position[RHGUARD] == 4) DrawObject(RHGUARD, 3, 141, 46, winPaint);
    else if(position[RHGUARD] == 5) DrawObject(RHGUARD, 4, 147, 26, winPaint);
    
    if(position[RHGUARD] > 0) play_sound(1);
    
    old_position[RHGUARD] = position[RHGUARD];
  }

  if(position[LHGUARD] != old_position[LHGUARD])
  {
    if(old_position[LHGUARD] == 1)      DrawObject(LHGUARD, 5, 52, 104, winPaint);
    else if(old_position[LHGUARD] == 2) DrawObject(LHGUARD, 6, 26, 99, winPaint);
    else if(old_position[LHGUARD] == 3) DrawObject(LHGUARD, 7, 17, 85, winPaint);
    else if(old_position[LHGUARD] == 4) DrawObject(LHGUARD, 9, 9, 50, winPaint);

    if(position[LHGUARD] == 1)      DrawObject(LHGUARD, 0, 52, 104, winPaint);
    else if(position[LHGUARD] == 2) DrawObject(LHGUARD, 1, 26, 99, winPaint);
    else if(position[LHGUARD] == 3) DrawObject(LHGUARD, 2, 17, 85, winPaint);
    else if(position[LHGUARD] == 4) DrawObject(LHGUARD, 3, 9, 64, winPaint);
    
    if(position[LHGUARD] > 0) play_sound(2);
        
    old_position[LHGUARD] = position[LHGUARD];
  }
  
  // draw the convict  
  if(position[CONVICT] != old_position[CONVICT])
  {
    // extend pickaxe
    if(position[CONVICT] == 9)
    {
      if(position[SEARCHLIGHT] == 5) DrawObject(CONVICT, 28, 35, 40, winPaint); // unstomp light
      else                           DrawObject(CONVICT,  8, 35, 40, winPaint);
    }
    // retract pickaxe
    else if(old_position[CONVICT] == 9)
    {
      if(position[SEARCHLIGHT] == 5) DrawObject(CONVICT, 27, 35, 40, winPaint); // unstomp light
      else                           DrawObject(CONVICT,  7, 35, 40, winPaint);
    }
    
    // "normal" positions require clearing one old position before
    // drawing convict in new position
    else if(old_position[CONVICT] == 0)                                DrawObject(CONVICT, 10, 127, 27, winPaint);
    else if(old_position[CONVICT] == 1 && position[SEARCHLIGHT] != 0)  DrawObject(CONVICT, 11, 124, 51, winPaint);
    else if(old_position[CONVICT] == 2)                                DrawObject(CONVICT, 12, 114, 75, winPaint);
    else if(old_position[CONVICT] == 3 && position[SEARCHLIGHT] != 2)  DrawObject(CONVICT, 13, 99, 76, winPaint);
    else if(old_position[CONVICT] == 4 && position[SEARCHLIGHT] != 3)  DrawObject(CONVICT, 14, 83, 81, winPaint);
    else if(old_position[CONVICT] == 5 && position[SEARCHLIGHT] != 4)  DrawObject(CONVICT, 15, 64, 77, winPaint);
    else if(old_position[CONVICT] == 6)                                DrawObject(CONVICT, 16, 41, 76, winPaint);
    else if(old_position[CONVICT] == 7 && position[SEARCHLIGHT] != 5)  DrawObject(CONVICT, 25, 48, 53, winPaint);
    else if(old_position[CONVICT] == 8)                                DrawObject(CONVICT, 17, 35, 40, winPaint);
    
    if(position[CONVICT] == 0)        DrawObject(CONVICT, 0, 127, 27, winPaint);
    else if(position[CONVICT] == 1)
    {
      if(position[SEARCHLIGHT] != 0)  DrawObject(CONVICT, 1, 124, 51, winPaint);  // unlit
      else                            DrawObject(CONVICT, 20, 125, 51, winPaint); // lit
    }
      
    else if(position[CONVICT] == 2)   DrawObject(CONVICT, 2, 114, 75, winPaint);
    else if(position[CONVICT] == 3)
    {
      if(position[SEARCHLIGHT] != 2)  DrawObject(CONVICT, 3, 99, 76, winPaint);  // unlit
      else                            DrawObject(CONVICT, 21, 99, 76, winPaint); // lit
    }
    else if(position[CONVICT] == 4)
    {
      if(position[SEARCHLIGHT] != 3)  DrawObject(CONVICT, 4, 83, 81, winPaint);  // unlit
      else                            DrawObject(CONVICT, 22, 83, 81, winPaint); // lit
    }
    else if(position[CONVICT] == 5)
    {
      if(position[SEARCHLIGHT] != 4)  DrawObject(CONVICT, 5, 64, 77, winPaint);  // unlit
      else                            DrawObject(CONVICT, 23, 64, 77, winPaint); // lit
    }
    else if(position[CONVICT] == 6)   DrawObject(CONVICT, 6, 41, 76, winPaint);
    else if(position[CONVICT] == 7)
    {
      if(position[SEARCHLIGHT] != 5)  DrawObject(CONVICT, 24, 48, 53, winPaint);  // unlit
      else                            DrawObject(CONVICT, 26, 48, 53, winPaint); // lit
    }
    else if(position[CONVICT] == 8)
    {
      if(position[SEARCHLIGHT] == 5)  DrawObject(CONVICT, 27, 35, 40, winPaint); // unstomp light
      else                            DrawObject(CONVICT,  7, 35, 40, winPaint);
    }

    old_position[CONVICT] = position[CONVICT];
  }
  

  // draw the holes in the wall  
  if(prefs.game_type == 0)
  {
    if(holes_dug == 1)      DrawObject(MISC, 1, 25, 30, winPaint);
    else if(holes_dug == 2) DrawObject(MISC, 2, 25, 30, winPaint);
    else if(holes_dug == 3) DrawObject(MISC, 3, 25, 30, winPaint);
  }
  if(prefs.game_type == 1)
  {
    if(holes_dug == 1)      DrawObject(MISC, 1, 25, 30, winPaint);
    else if(holes_dug == 3) DrawObject(MISC, 2, 25, 30, winPaint);
    else if(holes_dug == 5) DrawObject(MISC, 3, 25, 30, winPaint);
  }

  // draw captured prisoners
  if(num_captured_prisoners >= 1)
  {
    DrawObject(MISC, 6, 118, 131, winPaint);
  }
  if(num_captured_prisoners >= 2)
  {
    DrawObject(MISC, 6, 130, 131, winPaint);
  }
  if(num_captured_prisoners == 3)
  {
    DrawObject(MISC, 6, 142, 131, winPaint);
  }
  
  // wipe out the score box, then draw the score string
  DrawObject(MISC, 10, 7, 132, winPaint);
  WinDrawChars(score_string, StrLen(score_string), 9, 136);
}


/***********************************************************************
 ***********************************************************************/
static void reset_game(void)
{
  num_captured_prisoners = 0;
  score = 0;
  last_level_change = 0;

  game_speed_factor = 100;
  recalculate_intervals();  

  for(int i = 0; i < 4; i++)
  {
    position[i] = -1;
  }
  reset_positions();
  
  DrawBackground();
  DrawBoard();
}


/***********************************************************************
 ***********************************************************************/
static void sleep(int num_ticks)
{
  EventType event;

  long target_time = TimGetTicks() + num_ticks;
  do
  {
    EvtGetEvent (&event, num_ticks);
    
    if(TimGetTicks() >= target_time) break;
    
  } while (event.eType != appStopEvent);
}


/***********************************************************************
 ***********************************************************************/
 /*
static void animate_digging()
{
  // replace the convict icon with the "forward pick" icon
  DrawObject(CONVICT, 8, 35, 40, winPaint);
  play_sound(3);
  sleep(sysTicksPerSecond * 20 / 100);
  DrawObject(CONVICT, 7, 35, 40, winPaint);
  
  //back_off_events(sysTicksPerSecond * 20/100);
}
*/


/***********************************************************************
 ***********************************************************************/
static void animate_escape_sequence()
{
  DrawObject(CONVICT, 17, 35, 40, winPaint);
  for(int i = 0; i < 2; i++)
  {
    DrawObject(CONVICT, 9, 15, 19, winPaint);
    play_sound(5);
    sleep(sysTicksPerSecond * 15 / 100);
    play_sound(5);
    sleep(sysTicksPerSecond * 15 / 100);

    DrawObject(CONVICT, 19, 15, 19, winPaint);
    sleep(sysTicksPerSecond * 30 / 100);
  }
  
  DrawObject(MISC, 0, 25, 30, winPaint);
  
  back_off_events(sysTicksPerSecond * 120 / 100);

  return;
}


/***********************************************************************
 ***********************************************************************/
static void animate_capture_sequence()
{
  play_sound(4);
  for(int i = 0; i < 3; i++)
  {
    DrawObject(MISC, 4, 87, 102, winPaint);
    sleep(sysTicksPerSecond * 30 / 100);
    DrawObject(MISC, 5, 87, 102, winPaint);
    sleep(sysTicksPerSecond * 30 / 100);
  }
  
  back_off_events(sysTicksPerSecond * 180 / 100);
  
  return;
}


/***********************************************************************
 ***********************************************************************/
static void update_prisoners()
{
  num_captured_prisoners++;

  DrawBoard();
  if(num_captured_prisoners == 1)
  {
    reset_positions();
  }
  else if(num_captured_prisoners == 2)
  {
    reset_positions();
  }
  
  // game over
  else if(num_captured_prisoners == 3)
  {
    sleep(sysTicksPerSecond);
    DrawObject(MISC, 8, 46, 58, winPaint);
    sleep(sysTicksPerSecond);
    DrawObject(MISC, 9, 46, 58, winPaint);
    
    back_off_events(sysTicksPerSecond * 2);

    reset_convict();

    if(prefs.game_type == 0 && score > prefs.high_score_a)
    {
      prefs.high_score_a = score;
    }
    else if(prefs.game_type == 1 && score > prefs.high_score_b)
    {
      prefs.high_score_b = score;
    }    
    // fake the old position so that the DrawBoard() routine will
    // redraw the searchlight at its current position
    old_position[SEARCHLIGHT] = -1;
    prefs.demo_mode = 1;
  }

  DrawBoard();
}


/***********************************************************************
 ***********************************************************************/
static void check_for_capture()
{
  // we don't worry about checking when we're running in demo mode
  if(prefs.demo_mode == 1) return;
  
  // caught by RH guard
  if(position[CONVICT] != 0 && position[RHGUARD] == 5)
  {
    sleep(sysTicksPerSecond);
    animate_capture_sequence();
    update_prisoners();
  }
  // caught by searchlight
  if((position[CONVICT] == 1 && position[SEARCHLIGHT] == 0)
     || (position[CONVICT] == 3 && position[SEARCHLIGHT] == 2)
     || (position[CONVICT] == 4 && position[SEARCHLIGHT] == 3)
     || (position[CONVICT] == 5 && position[SEARCHLIGHT] == 4)
     || (position[CONVICT] == 7 && position[SEARCHLIGHT] == 5))
  {
    sleep(sysTicksPerSecond);
    animate_capture_sequence();
    position[CONVICT] = -1;   // hack to keep from displaying the "convict off" bitmap
    update_prisoners();
  }
  // caught by LH guard
  if(position[CONVICT] == 8 && position[LHGUARD] == 4)
  {
    sleep(sysTicksPerSecond * 25 / 100);
    DrawObject(LHGUARD, 4, 9, 50, winPaint);
    sleep(sysTicksPerSecond);
    animate_capture_sequence();
    update_prisoners();
  }
}


/***********************************************************************
 ***********************************************************************/
static void MoveConvict(UInt32 hardKeyState)
{
  if (in_act_of_digging == 1) return;
  
  if (hardKeyState & moveLeftKeys && !(hardKeyState & moveRightKeys))
  {
    if(position[CONVICT] < 8)
    {
      old_position[CONVICT] = position[CONVICT];
      position[CONVICT]++;
      
      if(position[CONVICT] > farthest_position)
      {
        score++;
        farthest_position = position[CONVICT];
      }
    }
   
    // if we're at the wall, the left key digs the hole;
    // game type A requires 3 digs to get out, type B requires 5
    else if(position[CONVICT] == 8)
    {
      if((prefs.game_type == 0 && holes_dug == 3)
         || (prefs.game_type == 1 && holes_dug == 5))
      {
        // convict is free
        animate_escape_sequence();
        score += 5;

        // speed up the game every 50 points
        if(score - last_level_change > 50)
        {
          if(game_speed_factor >= min_game_speed_factor + 7)
          {
            game_speed_factor -= 7;
            recalculate_intervals();
          }  
          last_level_change = score;  
        }  
                
        // don't reset everything -- just the convict
        //reset_positions();
        reset_convict();
        DrawBoard();
      }
      else
      {
        old_position[CONVICT] = 8;
        position[CONVICT] = 9;    // pseudo-position when pickaxe is in digging position
        time_of_next_event[CONVICT] = TimGetTicks() + sysTicksPerSecond * 20 / 100;
        in_act_of_digging = 1;
        //        animate_digging();
        holes_dug++;
        score++;
        play_sound(3);
      }  
    }
  }
  
  if (hardKeyState & moveRightKeys && !(hardKeyState & moveLeftKeys))
  {
    if(position[CONVICT] >0)
    {
      old_position[CONVICT] = position[CONVICT];
      position[CONVICT]--;
    }
  }
  
  DrawBoard();
  check_for_capture();
}


/***********************************************************************
 ***********************************************************************/
static void AdvanceGameState(void)
{
  if(game_paused) return;
  
  long current_time = TimGetTicks();
  
  for(int i = 0; i < 3; i++)
  {
    if(time_of_next_event[i] > 0
       && current_time >= time_of_next_event[i])
    {
      old_position[i] = position[i];
  
      if(direction[i] == 1)
      {
        if(position[i] < num_positions[i] - 1) position[i]++;
        else
        {
          direction[i] = 0;
          position[i]--;
        }
      }
      else if(direction[i] == 0)
      {
        if(position[i] > 0) position[i]--;
        else
        {
          direction[i] = 1;
          position[i]++;
        }
      }
    
      time_of_next_event[i] = get_next_interval(i, time_of_next_event[i], position[i]);
    }
  }
  if(time_of_next_event[CONVICT] > 0
       && current_time >= time_of_next_event[CONVICT])
  {
    old_position[CONVICT] = 9;
    position[CONVICT] = 8;
    in_act_of_digging = 0;
    time_of_next_event[CONVICT] = 0;
  }
  
  find_soonest_event();
  DrawBoard();
  check_for_capture();
}


/***********************************************************************
 ***********************************************************************/
static void SearchlightInit(void)
{
   UInt16 error;
   
   num_positions[SEARCHLIGHT] = 6;
   num_positions[RHGUARD] = 6;
   num_positions[LHGUARD] = 5;
   
   num_bitmaps[SEARCHLIGHT] = 12;
   num_bitmaps[RHGUARD] = 10;
   num_bitmaps[LHGUARD] = 10;
   num_bitmaps[CONVICT] = 29;
   num_bitmaps[MISC]    = 11;
   
   resource_base[SEARCHLIGHT] = 5010;
   resource_base[RHGUARD] = 6000;
   resource_base[LHGUARD] = 7000;
   resource_base[CONVICT] = 8000;
   resource_base[MISC]    = 9000;

   // load all the bitmaps
   WinHandle oldDrawWinH = WinGetDrawWindow();
   
   bgBitmapHandle = DmGetResource(bitmapRsc, 5000);

   bgBitmapPtr = (BitmapPtr) MemHandleLock(bgBitmapHandle);
   bgWindowHandle = WinCreateOffscreenWindow(bgBitmapPtr->width, bgBitmapPtr->height, screenFormat, &error);

   static char error_string[255];
   StrPrintF(error_string, "Error loading resource 5000");
   ErrFatalDisplayIf(error, error_string);

   WinSetDrawWindow(bgWindowHandle);
   WinDrawBitmap(bgBitmapPtr, 0, 0);
   
   // load sprite bitmaps
   for(int i = 0; i < 5; i++)
   {
     if(resource_base[i] > 0)
     {
       for (int j = 0; j < num_bitmaps[i]; j++)
       {
         ObjectBitmapHandles[i][j] = DmGetResource(bitmapRsc, resource_base[i] + 10 * j);
         ObjectBitmapPtr[i][j] = (BitmapPtr) MemHandleLock(ObjectBitmapHandles[i][j]);
		
         ObjectWindowHandles[i][j] = WinCreateOffscreenWindow(
			ObjectBitmapPtr[i][j]->width, ObjectBitmapPtr[i][j]->height,
			screenFormat, &error);

         static char error_string[255];
         StrPrintF(error_string, "Error loading sprite %d, bitmap %d, resource %d", i, j, resource_base[i] + 10 * j);
         ErrFatalDisplayIf(error, error_string);

	     WinSetDrawWindow(ObjectWindowHandles[i][j]);
	     WinDrawBitmap(ObjectBitmapPtr[i][j], 0, 0);
	   }
	 }
   }
   WinSetDrawWindow(oldDrawWinH);
   
   sound_volume = PrefGetPreference(prefGameSoundVolume);
   
   // if we're in demo mode, just reset everything
   if(prefs.demo_mode == 1)
   {
     reset_game();
   }
   
   // otherwise, pick up the game where we left off.
   else
   {
     recalculate_intervals();
     
     long current_time = TimGetTicks();
     for(int i = 0; i < 3; i++)
     {
       old_position[i] = -1;
       time_of_next_event[i] = get_next_interval(i, current_time, position[i]);
   /*    
  long begin_time = TimGetTicks();
  static char msg_string[255];
  StrPrintF(msg_string, "sprite %d, position: %d, next_event: %ld (%ld)", i, position[i], time_of_next_event[i], current_time);
  FrmCustomAlert(GenericErrorAlert, msg_string, NULL, NULL);
  long push_back = TimGetTicks() - begin_time;
  back_off_events((int)push_back);
*/
     }
     time_of_next_event[CONVICT] = 0;
     find_soonest_event();
     old_position[CONVICT] = -1;
   }
 
   DrawBackground();
   DrawBoard();
}


/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(UInt16 command)
{
  Boolean handled = false;
  FormPtr frmP;

  switch (command)
  {
 	case MainGameAbout:
 	case GameAbout:
			MenuEraseStatus(0);
			frmP = FrmInitForm (AboutForm);
			FrmDoDialog (frmP);					// Display the About Box.
			FrmDeleteForm (frmP);
			handled = true;
			break;

 	case MainGameOptions:
 	case GameOptions:
	        if (game_paused == 0) toggle_pause();

			MenuEraseStatus(0);
			FrmPopupForm(OptionsForm);
			
			/*
			FrmDoDialog (frmP);					// Display the Options form
			FrmDeleteForm (frmP);
			*/
			
			handled = true;
			break;
			
	case MainGamePauseGame:
	        if(game_paused == 0) toggle_pause();
	        user_paused = 1;
            FrmSetMenu(FrmGetFormPtr(MainForm), MainForm2MenuBar);
	        handled = true;
	        break;		
	
	case GameResumeGame:
	        if(game_paused == 1) toggle_pause();
	        user_paused = 0;
            FrmSetMenu(FrmGetFormPtr(MainForm), MainFormMenuBar);
	        handled = true;
	        break;		
	        
	case MainGameNewGame:
	case GameNewGame:
         	if(game_paused == 1) toggle_pause();

            if(game_paused == 0) FrmSetMenu(FrmGetFormPtr(MainForm), MainFormMenuBar);
            else                 FrmSetMenu(FrmGetFormPtr(MainForm), MainForm2MenuBar);

	        prefs.demo_mode = 0;
	        reset_game();
	        handled = true;
	        break;
			
  }
	
  return handled;
}


/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr eventP)
{
   Boolean handled = false;
   FormPtr frmP;

	switch (eventP->eType) 
	{
	    case winExitEvent:
	        if (eventP->data.winExit.exitWindow == (WinHandle) FrmGetFormPtr(MainForm)
	             && game_paused == 0)
	             {
	               toggle_pause();
	             }  
	        break;
	        
	    case winEnterEvent:
	        if (eventP->data.winEnter.enterWindow == (WinHandle) FrmGetFormPtr(MainForm)
	            && eventP->data.winEnter.enterWindow == (WinHandle) FrmGetFirstForm()
	            && game_paused == 1 && user_paused != 1) 
	            {
	              toggle_pause();
	            }
	        break;
	        
		case menuEvent:
			return MainFormDoCommand(eventP->data.menu.itemID);

		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			MainFormInit(frmP);
			FrmDrawForm (frmP);
			handled = true;
			break;

		default:
			break;
		
	}
	
	return handled;
}


/***********************************************************************
 ***********************************************************************/
static Boolean OptionsFormHandleEvent(EventPtr eventP)
{
   Boolean handled = false;
   FormPtr frmP;

   switch (eventP->eType) 
		{
		case menuEvent:
			return MainFormDoCommand(eventP->data.menu.itemID);
			
/*
        case popSelectEvent:
            if(eventP->eType.data.controlID == OptionsDifficultyPopTrigger)
            {
              game_difficulty = eventP->data.popSelect.selection;
              handled = true;
            }

            break;
  */      
        /*
        case ctlSelectEvent:
            if(eventP->data.controlID == OptionsOKButton)
            {
              // close the options form
              
              handled = true;
            }*/

        case ctlSelectEvent:
            switch (eventP->data.ctlSelect.controlID)
            {
                 case OptionsGameTypeAPushButton:
                     CtlSetValue((ControlPtr)GetObjectPtr(OptionsGameTypeBPushButton), 0);
                     prefs.game_type = 0;
                     break;               
                 case OptionsGameTypeBPushButton:
                     CtlSetValue((ControlPtr)GetObjectPtr(OptionsGameTypeAPushButton), 0);
                     prefs.game_type = 1;
                     break;
                 case OptionsSoundCheckbox:
                     prefs.sound_option = CtlGetValue((ControlPtr)GetObjectPtr(OptionsSoundCheckbox));
                     break;

     	        case OptionsOKButton:
                     FrmReturnToForm(0);
                     if (game_paused == 1 && user_paused != 1) toggle_pause();

                     break;
            }
            
            handled = true;
            break;
               
		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			FrmDrawForm(frmP);
			
			if(prefs.game_type == 1)
			{
              CtlSetValue((ControlPtr)GetObjectPtr(OptionsGameTypeAPushButton), 0);
              CtlSetValue((ControlPtr)GetObjectPtr(OptionsGameTypeBPushButton), 1);
			}
			else
			{
              CtlSetValue((ControlPtr)GetObjectPtr(OptionsGameTypeAPushButton), 1);
              CtlSetValue((ControlPtr)GetObjectPtr(OptionsGameTypeBPushButton), 0);
			}

            CtlSetValue((ControlPtr)GetObjectPtr(OptionsSoundCheckbox), prefs.sound_option);
			
			handled = true;
			break;

		default:
			break;
		
		}
	
	return handled;
}



/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventPtr eventP)
{
	UInt16 formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent)
		{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)
			{
			case MainForm:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;
				
			case OptionsForm:
			    FrmSetEventHandler(frmP, OptionsFormHandleEvent);
			    break;

			default:
//				ErrFatalDisplay("Invalid Form Load Event");
				break;

			}
		return true;
		}
	
	return false;
}

/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do
		{
		// Wait until the next game period.
		//EvtGetEvent (&event, evtWaitForever);

		EvtGetEvent (&event, TimeUntilNextPeriod());

		// Detect exiting the game's window.  This must be checked for.  At this
		// point there probably exists another window which may cover part of
		// the BoardView window.  Suppress drawing.  Otherwise drawing may draw
		// to part of the window covered by the new window.
		
		// we also have a check for this in the MainFormHandleEvent() function;
		// I'm not sure that we need both of them.
		if (event.eType == winExitEvent)
		{
		  if (event.data.winExit.exitWindow == (WinHandle) FrmGetFormPtr(MainForm)
			    && game_paused == 0)
	      {
	        toggle_pause();
	      }  
		}


		// Intercept the hard keys to prevent them from switching apps
		if (event.eType == keyDownEvent && 
			event.data.keyDown.chr >= hard1Chr &&
			event.data.keyDown.chr <= hard4Chr &&
			!(event.data.keyDown.modifiers & poweredOnKeyMask) &&
			game_paused == 0)
		{
		  if(prefs.demo_mode == 1)
		  {
  	        prefs.demo_mode = 0;
	        reset_game();
		  }
		  else
		  {
            UInt32 hardKeyState = KeyCurrentState() & keysAllowedMask;
            MoveConvict(hardKeyState);
          }  
          continue;
		}

		if(TimeUntilNextPeriod() == 0)
		{
   	      AdvanceGameState();
		}
		

		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}




/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value 0 if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
  UInt16 prefsSize;

  // Read the saved preferences / saved-state information.
  prefsSize = sizeof(SearchlightPreferenceType);
  if (PrefGetAppPreferences(appFileCreator, appPrefID, &prefs, &prefsSize, true) != 
	noPreferenceFound)
  {
     if(prefs.demo_mode == 0)
     {
       position[LHGUARD]      = prefs.lhguard_position;
       direction[LHGUARD]     = prefs.lhguard_direction;
       position[SEARCHLIGHT]  = prefs.searchlight_position;    
       direction[SEARCHLIGHT] = prefs.searchlight_direction;
       position[RHGUARD]      = prefs.rhguard_position;
       direction[RHGUARD]     = prefs.rhguard_direction;
       position[CONVICT]      = prefs.convict_position;
       farthest_position      = prefs.farthest_position;
       num_captured_prisoners = prefs.num_captured_prisoners;
       holes_dug              = prefs.holes_dug;
       score                  = prefs.score;
       game_speed_factor      = prefs.game_speed_factor;
       last_level_change      = prefs.last_level_change;
     }
  }
  else
  {
     prefs.game_type       = 0;
     prefs.game_difficulty = 2;
     prefs.sound_option    = 1;
     prefs.high_score_a    = 0;
     prefs.high_score_b    = 0;
     prefs.demo_mode       = 1;
  }

  return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
  if(prefs.demo_mode == 0)
  {
    prefs.lhguard_position       = position[LHGUARD];
    prefs.lhguard_direction      = direction[LHGUARD];
    prefs.searchlight_position   = position[SEARCHLIGHT];
    prefs.searchlight_direction  = direction[SEARCHLIGHT];
    prefs.rhguard_position       = position[RHGUARD];
    prefs.rhguard_direction      = direction[RHGUARD];
    prefs.convict_position       = position[CONVICT];
    prefs.farthest_position      = farthest_position;
    prefs.num_captured_prisoners = num_captured_prisoners;
    prefs.holes_dug              = holes_dug;
    prefs.score                  = score;
    prefs.game_speed_factor      = game_speed_factor;
    prefs.last_level_change      = last_level_change;
  }
  
  // Write the saved preferences / saved-state information.  This data 
  // will be backed up during a HotSync.
  PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		&prefs, sizeof (prefs), true);
		
  // Close all the open forms.
  FrmCloseAllForms ();
}


/***********************************************************************
 ***********************************************************************/
static UInt32 SearchlightPalmMain(UInt16 cmd, MemPtr /*cmdPBP*/, UInt16 launchFlags)
{
	Err error;

	error = RomVersionCompatible (ourMinVersion, launchFlags);
	if (error) return (error);

	switch (cmd)
		{
		case sysAppLaunchCmdNormalLaunch:
            UInt32 requiredDepth = 2; 
            Err err = WinScreenMode(winScreenModeSet, NULL, NULL, &requiredDepth, NULL);
            ErrFatalDisplayIf(err, "Cannot set screen to 2-bit depth.");
 
			error = AppStart();
			if (error) 
				return error;
				
			FrmGotoForm(MainForm);

	        SearchlightInit();

			AppEventLoop();
			AppStop();
			break;

		default:
			break;

		}
	
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    return SearchlightPalmMain(cmd, cmdPBP, launchFlags);
}

